#include "MJX_v1.h"
#include "CC2500.h"

// This sequence is used by any TX during bind phase
uint8_t rf_channels_bind[21] = {
  0x43, 0x03, 0x83, 0x23, 0x63, 0x0a, 0x56, 0x30,
  0x76, 0x16, 0x50, 0x8a, 0x36, 0x6a, 0x10, 0x50,
  0x2a, 0x7d, 0x4a, 0x1d, 0x70
};

// This sequence is used by MiniPET TX with SEQ ID 0xED
uint8_t rf_channels_ed[21] = {
  0x41, 0x74, 0x21, 0x4E, 0x81, 0x27, 0x54, 0x14,
  0x6E, 0x3A, 0x8E, 0x61, 0x1A, 0x7a, 0x34, 0x5A,
  0x0E, 0x2E, 0x67, 0x87, 0x47
};



// First one is random byte (on T40C RX) which determines freq hopping sequence
// Second and third looks like TX Type ( 6D 9B on T40C, 6D CF on MiniPET TX)

void MJX_TX::setTXId(uint8_t txid_[3])
{
/*
  txid[0] = txid_[0];
  txid[1] = txid_[1];
  txid[2] = txid_[2];
*/
  // MiniPET TX
  txid[0] = 0xED;
  txid[1] = 0x6D;
  txid[2] = 0xCF;
}

void MJX_TX::begin()
{
  // INIT SEQUENCE
  Serial.print("Init TX\n");
  radio.init();
  radio.sendCommand(CC2500_OFF_WRITE_SINGLE | CC2500_REG_CHANNR, 0x59);
  radio.sendCommand(CC2500_OFF_WRITE_SINGLE | CC2500_REG_ADDR, 0xAA);
  radio.sendStrobeCommand(CC2500_CMD_SRES);
  radio.sendCommand(CC2500_OFF_WRITE_SINGLE | CC2500_REG_IOCFG2, 0x09); // GDO2: Clear channel assessment. High when RSSI level is below threshold
  radio.sendCommand(CC2500_OFF_WRITE_SINGLE | CC2500_REG_IOCFG1, 0x2E); // GDO1: High impedance (3-state)
  radio.sendCommand(CC2500_OFF_WRITE_SINGLE | CC2500_REG_IOCFG0, 0x06); // GDO0: Asserts when sync word has been sent / received, and de-asserts at the end of the packet.
  radio.sendCommand(CC2500_OFF_WRITE_SINGLE | CC2500_REG_FIFOTHR, 0x07);
  radio.sendCommand(CC2500_OFF_WRITE_SINGLE | CC2500_REG_SYNC1, 0xD3);
  radio.sendCommand(CC2500_OFF_WRITE_SINGLE | CC2500_REG_SYNC0, 0x91);
  radio.sendCommand(CC2500_OFF_WRITE_SINGLE | CC2500_REG_PKTLEN, 0x10);
  radio.sendCommand(CC2500_OFF_WRITE_SINGLE | CC2500_REG_PKTCTRL1, 0x04);
  radio.sendCommand(CC2500_OFF_WRITE_SINGLE | CC2500_REG_PKTCTRL0, 0x44);
  radio.sendCommand(CC2500_OFF_WRITE_SINGLE | CC2500_REG_ADDR, 0x00);
  radio.sendCommand(CC2500_OFF_WRITE_SINGLE | CC2500_REG_CHANNR, 0x00);
  radio.sendCommand(CC2500_OFF_WRITE_SINGLE | CC2500_REG_FSCTRL1, 0x09);
  radio.sendCommand(CC2500_OFF_WRITE_SINGLE | CC2500_REG_FSCTRL0, 0x00);
  radio.sendCommand(CC2500_OFF_WRITE_SINGLE | CC2500_REG_FREQ2, 0x5C);
  radio.sendCommand(CC2500_OFF_WRITE_SINGLE | CC2500_REG_FREQ1, 0xCE);
  radio.sendCommand(CC2500_OFF_WRITE_SINGLE | CC2500_REG_FREQ0, 0xC4);
  radio.sendCommand(CC2500_OFF_WRITE_SINGLE | CC2500_REG_MDMCFG4, 0x2C);
  radio.sendCommand(CC2500_OFF_WRITE_SINGLE | CC2500_REG_MDMCFG3, 0xB4);
  radio.sendCommand(CC2500_OFF_WRITE_SINGLE | CC2500_REG_MDMCFG2, 0x73);
  radio.sendCommand(CC2500_OFF_WRITE_SINGLE | CC2500_REG_MDMCFG1, 0x23);
  radio.sendCommand(CC2500_OFF_WRITE_SINGLE | CC2500_REG_MDMCFG0, 0xFE);
  radio.sendCommand(CC2500_OFF_WRITE_SINGLE | CC2500_REG_DEVIATN, 0x01);
  radio.sendCommand(CC2500_OFF_WRITE_SINGLE | CC2500_REG_MCSM2, 0x07);
  radio.sendCommand(CC2500_OFF_WRITE_SINGLE | CC2500_REG_MCSM1, 0x30);
  radio.sendCommand(CC2500_OFF_WRITE_SINGLE | CC2500_REG_MCSM0, 0x08);
  radio.sendCommand(CC2500_OFF_WRITE_SINGLE | CC2500_REG_FOCCFG, 0x1D);
  radio.sendCommand(CC2500_OFF_WRITE_SINGLE | CC2500_REG_BSCFG, 0x1C);
  radio.sendCommand(CC2500_OFF_WRITE_SINGLE | CC2500_REG_AGCCTRL2, 0xC7);
  radio.sendCommand(CC2500_OFF_WRITE_SINGLE | CC2500_REG_AGCCTRL1, 0x00);
  radio.sendCommand(CC2500_OFF_WRITE_SINGLE | CC2500_REG_AGCCTRL0, 0xB2);
  radio.sendCommand(CC2500_OFF_WRITE_SINGLE | CC2500_REG_WOREVT1, 0x87);
  radio.sendCommand(CC2500_OFF_WRITE_SINGLE | CC2500_REG_WOREVT0, 0x6B);
  radio.sendCommand(CC2500_OFF_WRITE_SINGLE | CC2500_REG_WORCTRL, 0xF8);
  radio.sendCommand(CC2500_OFF_WRITE_SINGLE | CC2500_REG_FREND1, 0xB6);
  radio.sendCommand(CC2500_OFF_WRITE_SINGLE | CC2500_REG_FREND0, 0x10);
  radio.sendCommand(CC2500_OFF_WRITE_SINGLE | CC2500_REG_FSCAL3, 0xEA);
  radio.sendCommand(CC2500_OFF_WRITE_SINGLE | CC2500_REG_FSCAL2, 0x0A);
  radio.sendCommand(CC2500_OFF_WRITE_SINGLE | CC2500_REG_FSCAL1, 0x00);
  radio.sendCommand(CC2500_OFF_WRITE_SINGLE | CC2500_REG_FSCAL0, 0x11);
  radio.sendCommand(CC2500_OFF_WRITE_SINGLE | CC2500_REG_RCCTRL1, 0x41);

  uint8_t patable_data[] = { 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff };
  radio.sendBurstCommand(CC2500_OFF_WRITE_BURST | CC2500_REG_PATABLE , patable_data, sizeof(patable_data));
  Serial.print("Init done\n");
/* 
  Serial.print(" bind channels: \n");
  for (int i=0;i<21;i++) {
    Serial.print(rf_channels_bind[i]);
    Serial.print(" ");
  }
  Serial.print("\n");
*/
  rf_ch_num = 0;
  counter = 0;
}



// Payload
// 56 00 80 80 80 28 28 28 6D 9B 00 00 00 00 08 48
// SS TH RU EL 80 TR TE 28 TX ID 00 00 00 00 ST CS
//
// SS: Channel Sequence ID
// TX ID: 6D 9B on T40C, 6D CF on MiniPET TX
// Throttle: 0 = min, all other 0x80 = center
// Trim values: 0x28 = center
// ST: Status  0xC0 -> bind, 0x10 -> Camera

void MJX_TX::command(uint8_t throttle, int8_t yaw, int8_t pitch, int8_t roll, uint8_t flags)
{
  //Serial.print("Sending packet\n");
  uint8_t buf[16];
  if (flags == 0xc0) {
    // binding
    buf[1] = 0x00;
    buf[2] = 0x80;
    buf[3] = 0x80;
    buf[4] = 0x80;
  } else {
    // regular packet
    buf[1] = throttle;
  }
  buf[2] = (uint8_t) yaw;
  buf[3] = (uint8_t) roll;
  buf[4] = (uint8_t) pitch; // negativ = forward
  // Trims, middle is 0x28
  buf[5] = 0x28; // yaw
  buf[6] = 0x28; // pitch
  buf[7] = 0x28; // roll
  
  // TX id
  buf[0] = txid[0]; // channel sequence
  buf[8] = txid[1];
  buf[9] = txid[2];
  // empty
  buf[10] = 0x00;
  buf[11] = 0x00;
  buf[12] = 0x00;
  buf[13] = 0x00;
  //
  buf[14] = flags;
  uint8_t sum = 0;
  uint8_t i;
  for (i = 0; i < 15;  ++i) sum += buf[i];
  buf[15] = sum;

  // Print actual payload once per second
  counter++;
  if (counter == 125) {
    counter = 0;
    Serial.print(" Send: ");
    for (int i=0;i<16;i++) { 
      if (buf[i]<0x10) Serial.print("0");
      Serial.print(buf[i],HEX); 
      Serial.print(" ");
    }
    Serial.print("\n");
  }

  // Stop transmission
  radio.sendStrobeCommand(CC2500_CMD_SIDLE);
  
  // Flush TX Fifo
  radio.sendStrobeCommand(CC2500_CMD_SFTX);
  
  // Push payload into TX Fifo
  radio.sendBurstCommand(CC2500_OFF_WRITE_BURST | CC2500_REG_TXFIFO, buf, sizeof(buf));

  // Start transmission
  radio.sendStrobeCommand(CC2500_CMD_STX);

  // calculate new channel
  uint8_t rf_ch = (flags & 0xc0) ? rf_channels_bind[rf_ch_num] : rf_channels_ed[rf_ch_num];
/*  
  Serial.print(" rf_ch "); Serial.print(rf_ch);
  Serial.print(" rf_ch_num "); Serial.print(rf_ch_num);
  Serial.print("\n");  
*/  
  rf_ch_num++;
  if (rf_ch_num >= 21) rf_ch_num = 0;
  
  // Wait for GDO0 get one, then zero again (-> Packet transmission)
  uint8_t pktstatus;
  do {
    pktstatus = radio.sendCommand(CC2500_OFF_READ_BURST | CC2500_REG_PKTSTATUS, 0x00);
  } while (!(pktstatus & 1));
  do {
    pktstatus = radio.sendCommand(CC2500_OFF_READ_BURST | CC2500_REG_PKTSTATUS, 0x00);
  } while (pktstatus & 1);
  
  
  // Now switch to new channel
  radio.sendStrobeCommand(CC2500_CMD_SIDLE);

  radio.sendCommand(CC2500_OFF_WRITE_SINGLE | CC2500_REG_CHANNR, rf_ch);

  radio.sendStrobeCommand(CC2500_CMD_SCAL);
}


